/*
 * Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 * Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package edu.ucsc.its.temerity.shared.viewmodel

import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.setValue
import androidx.lifecycle.viewModelScope
import edu.ucsc.its.temerity.core.Temerity
import edu.ucsc.its.temerity.shared.viewmodel.AppStateEvent.ServiceEndpointUpdated
import edu.ucsc.its.temerity.shared.viewmodel.AppStateEvent.ServiceTokenUpdated
import edu.ucsc.its.temerity.shared.viewmodel.AppStateEvent.UserLoggedIn
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import org.koin.core.component.KoinComponent
import org.koin.core.component.get
import org.koin.core.parameter.parametersOf

sealed interface AppStateEvent {
  data class ServiceEndpointUpdated(val newEndpoint: String) : AppStateEvent
  data class ServiceTokenUpdated(val newToken: String) : AppStateEvent
  data class UserLoggedIn(val newStatus: Boolean) : AppStateEvent
}

data class AppState(
  val serviceEndpoint: String = "",
  val serviceToken: String = "",
  val userLoggedIn: Boolean = false,
)

class AppViewModel :
  MoleculeViewModel<AppStateEvent, AppState>(),
  KoinComponent {
  private var serviceEndpoint by mutableStateOf("")
  private var serviceToken by mutableStateOf("")
  private var userLoggedIn by mutableStateOf(false)

  @Composable
  override fun models(events: Flow<AppStateEvent>): AppState {
    return presentAppState(events, get { parametersOf() })
  }

  @Composable
  private fun presentAppState(events: Flow<AppStateEvent>, temerity: Temerity): AppState {
    LaunchedEffect(Unit) {
      events.collect { event ->
        when (event) {
          is ServiceEndpointUpdated -> {
            viewModelScope.launch {
              serviceEndpoint = event.newEndpoint
            }
          }
          is ServiceTokenUpdated -> {
            viewModelScope.launch {
              serviceToken = event.newToken
            }
          }
          is UserLoggedIn -> {
            viewModelScope.launch {
              attemptLogin()
            }
          }
        }
      }
    }
    return AppState(serviceEndpoint, serviceToken, userLoggedIn)
  }

  private fun attemptLogin() {
  }
}
