/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.core

import edu.ucsc.its.temerity.TemClientConfig
import edu.ucsc.its.temerity.core.TemerityLibrary.koin
import io.ktor.client.HttpClient
import io.ktor.client.HttpClientConfig
import io.ktor.client.engine.HttpClientEngine
import io.ktor.client.plugins.HttpTimeout
import io.ktor.client.plugins.defaultRequest
import io.ktor.client.plugins.logging.LogLevel
import io.ktor.client.plugins.logging.Logging
import io.ktor.client.request.header
import io.ktor.http.URLProtocol
import co.touchlab.kermit.Logger as KermitLogger
import io.ktor.client.plugins.logging.Logger as KtorLogger
/**
 * Factory providing web request clients for the Temerity client.
 * Adapted from the tmdb-api project: https://github.com/MoviebaseApp/tmdb-kotlin/raw/refs/heads/main/tmdb-api/src/commonMain/kotlin/app/moviebase/tmdb/core/HttpClientFactory.kt
 */
internal object HttpClientFactory {
  /**
   * Adapted from https://github.com/joreilly/FantasyPremierLeague/
   * This function creates an HttpClient with the provided HttpClientEngine.
   * The HttpClientEngine is injected by the Koin platformModule(), which as its name suggests provides an HttpClientEngine implementation that is platform-dependent.
   * It configures the client to log all requests and responses if enableNetworkLogs is true. TODO: read this as a build setting
   * It also sets a default request header with the provided authToken.
   * @param httpClientEngine The HttpClientEngine to use for the HttpClient.
   * @param timeoutDuration The duration in milliseconds to wait for a response before timing out.
   * @param config A block specifying the following options:
   *  - Whether to log all requests and responses.
   *  - Service URL to make requests to.
   *  - Token to set as a default request header.
   * @return The created HttpClient.
   */
  internal fun buildHttpClient(
    httpClientEngine: HttpClientEngine,
    timeoutDuration: Long? = TemerityLibrary.DEFAULT_TIMEOUT,
    config: TemClientConfig,
  ): HttpClient {
    val defaultConfig: HttpClientConfig<*>.() -> Unit = {
      // Can't use install(ContentNegotiation){ json() } here because YuJa's API returns a 406 error if the Accept header is set to application/json
      // This requires JSON transformations to be done via manually-called library functions

      // Pass the stored authToken as a header in every request
      defaultRequest {
        header("authToken", config.serviceToken)
        url {
          protocol = URLProtocol.HTTPS
        }
      }

      if (config.optDebugEnabled) {
        if (config.httpClientLoggingBlock == null) {
          install(Logging) {
            val kermit = koin.get<KermitLogger>()
            logger = object : KtorLogger {
              override fun log(message: String) {
                kermit.d(message)
              }
            }
            level = LogLevel.ALL
            sanitizeHeader { headerKey ->
              headerKey == "authToken"
            }
          }
        } else {
          config.httpClientLoggingBlock?.let {
            Logging(it)
          }
        }
      }

      expectSuccess = config.expectSuccess

      if (config.useTimeout) {
        install(HttpTimeout) {
          connectTimeoutMillis = timeoutDuration
          requestTimeoutMillis = timeoutDuration
          socketTimeoutMillis = timeoutDuration
        }
      }

      config.httpClientConfigBlock?.invoke(this)
    }
    return config.httpClientBuilder?.invoke()?.config(defaultConfig) ?: HttpClient(httpClientEngine, defaultConfig)
  }
}
