/*
 *     Designed and developed in 2022-2024 by William Walker (wnwalker@ucsc.edu)
 *     Copyright 2022-2024 The Regents of the University of California. All rights reserved.
 *
 *     This library is free software; you can redistribute it and/or
 *     modify it under the terms of the GNU Lesser General Public
 *     License as published by the Free Software Foundation; version 2.1 of the License.
 *
 *     This library is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *     Lesser General Public License for more details.
 *
 *     You should have received a copy of the GNU Lesser General Public
 *     License along with this library; if not, write to the Free Software
 *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
package edu.ucsc.its.temerity.di

import co.touchlab.kermit.NoTagFormatter
import co.touchlab.kermit.Severity
import co.touchlab.kermit.loggerConfigInit
import co.touchlab.kermit.platformLogWriter
import com.skydoves.sandwich.ktorfit.ApiResponseConverterFactory
import de.jensklingenberg.ktorfit.ktorfit
import edu.ucsc.its.temerity.TemClientConfig
import edu.ucsc.its.temerity.api.PlatformApi
import edu.ucsc.its.temerity.api.createPlatformApi
import edu.ucsc.its.temerity.core.HttpClientFactory.buildHttpClient
import edu.ucsc.its.temerity.core.TemerityLibrary.koin
import io.ktor.client.HttpClient
import org.koin.core.Koin
import org.koin.core.component.KoinComponent
import org.koin.core.parameter.parametersOf
import org.koin.dsl.module
import co.touchlab.kermit.Logger as KermitLogger

internal fun loggerModule() = module {
  factory<co.touchlab.kermit.Logger> { tag ->
    createLogger(tag.getOrNull())
  }
}

/**
 * This function provides the Koin module for the Temerity library.
 * It includes the platform module and provides factories for [HttpClient] and [PlatformApi].
 */
internal val libModule = module {
  factory { (config: TemClientConfig) ->
    buildHttpClient(
      httpClientEngine = get(),
      config = config,
    )
  }
  factory { (config: TemClientConfig) ->
    val client: HttpClient = get { parametersOf(config) }
    val ktorfit = ktorfit {
      config.serviceUrl?.let { baseUrl(it) }
      httpClient(client)
      converterFactories(ApiResponseConverterFactory.create())
    }
    ktorfit.createPlatformApi()
  }
}

private fun createLogger(tag: String?): KermitLogger = KermitLogger(
  config = loggerConfigInit(
    platformLogWriter(NoTagFormatter),
    minSeverity = Severity.Debug,
  ),
  tag = tag ?: "TemerityLib",
)

internal abstract class TemerityKoinComponent : KoinComponent {
  override fun getKoin(): Koin = koin
}
